==> media_twig_tools-1.0.3.tar.gz: media_twig_tools/src/Services/ImageBuilder.php <==
<?php

namespace Drupal\media_twig_tools\Services;

use Drupal\file\FileInterface;
use Drupal\image\ImageStyleInterface;
use Drupal\media_twig_tools\Config\MediaTwigToolsConfigInterface;
use Drupal\media_twig_tools\Generators\ImageGenerator;

/**
 * This class builds image from settings.
 *
 * @package Drupal\media_twig_tools\Services
 */
class ImageBuilder {

  /**
   * Nom du service.
   *
   * @const string
   */
  const SERVICE_NAME = 'media_twig_tools.image_builder';

  /**
   * Rules separator.
   *
   * @const string
   */
  const RULES_SEPARATOR = '@';

  /**
   * Conf.
   *
   * @var \Drupal\media_twig_tools\Config\MediaTwigToolsConfigInterface
   */
  protected $mediaTwigToolsConfig;

  /**
   * ImageBuilder constructor.
   *
   * @param \Drupal\media_twig_tools\Config\MediaTwigToolsConfigInterface $mediaTwigToolsConfig
   *   The config.
   */
  public function __construct(MediaTwigToolsConfigInterface $mediaTwigToolsConfig) {
    $this->mediaTwigToolsConfig = $mediaTwigToolsConfig;
  }

  /**
   * Retourne le singleton.
   *
   * @return static
   *   Le singleton.
   */
  public static function me() {
    return \Drupal::service(static::SERVICE_NAME);
  }

  /**
   * Return the url image from settings.
   *
   * @param string $fileId
   *   The file id.
   * @param array $srcSettings
   *   The source settings.
   * @param array $srcSetSettings
   *   The source set settings.
   * @param array $options
   *   The options.
   * @param string $rule
   *   The rule.
   *
   * @return array|string|string[]|null
   *   The image path.
   */
  public function getImageUrlFromSettings($fileId, array $srcSettings, array $srcSetSettings, array $options, $rule = 'reduce') {
    $imgUrl = NULL;

    // Get file.
    if ($file = $this->mediaTwigToolsConfig->getFileStorage()
      ->load(intval($fileId))) {

      $url = file_create_url($file->getFileUri());
      if ($options[MediaTwigTools::OPTION_RELATIVE]) {
        $url = file_url_transform_relative($url);
      }

      $imgUrl = $this->getImageUrlFromFileAndSettings($file->id(), $url, $srcSettings, $srcSetSettings, $options, $rule);
    }

    return $imgUrl;
  }

  /**
   * Quick replace.
   *
   * @param string $source
   *   The source string.
   * @param array $replace
   *   The replacement [$needle => $replacement].
   *
   * @return array|string|string[]
   *   The string replaced.
   */
  public function replace($source, array $replace) {
    return str_replace(
      array_keys($replace),
      array_values($replace),
      $source,
    );
  }

  /**
   * Return the rules from the dirname.
   *
   * @param string $path
   *   THe path.
   *
   * @return array
   *   The rules.
   */
  public function getRulesFromPathString($path) {
    $rules = [];
    foreach (explode(static::RULES_SEPARATOR, $path) as $data) {
      // Split letters and numbers.
      $split = preg_split("/(,?\s+)|((?<=[a-z])(?=\d))|((?<=\d)(?=[a-z]))/i", $data);

      // Associative array.
      if (is_numeric(reset($split))) {
        $key = 1;
        $delta = -1;
      }
      else {
        $key = 0;
        $delta = 1;
      }
      $rule = [
        'id'   => array_pop($split),
        'data' => [],
      ];
      for ($i = $key; $i <= count($split) - $delta; $i += 2) {
        if (isset($split[$i + $delta]) && isset($split[$i])) {
          $rule['data'][$split[$i]] = $split[$i + $delta];
        }
      }

      $rules[] = $rule;
    }

    return $rules;
  }

  /**
   * Return rules data.
   *
   * @param array $srcSettings
   *   The src settings.
   *
   * @return array
   *   The rules data.
   */
  public function getRulesData(array $srcSettings) {

    if (isset($srcSettings[MediaTwigTools::OPTION_STYLE])
      && $style = $this->mediaTwigToolsConfig->getImageStyleStorage()
        ->load($srcSettings[MediaTwigTools::OPTION_STYLE])) {
      $srcSettings[MediaTwigTools::OPTION_STYLE] = $style;
    }
    else {
      unset($srcSettings[MediaTwigTools::OPTION_STYLE]);
    }

    return $srcSettings;
  }

  /**
   * Return the dirname according to rules data.
   *
   * @param array $rulesData
   *   The rules data.
   *
   * @return string
   *   The rules dirname.
   */
  public function getRulesDirnameFromRulesData(array $rulesData) {
    $rules = [];
    foreach ($rulesData[MediaTwigTools::OPTION_RULES] as $weight => $rule) {
      $rules[$weight] = '';
      foreach ($rule['data'] as $key => $value) {
        $rules[$weight] .= $key . $value;
      }
      $rules[$weight] .= $rule['id'];
    }

    return implode(static::RULES_SEPARATOR, $rules);
  }

  /**
   * Return true if $path is an image builder route.
   *
   * @param string $path
   *   THe path.
   *
   * @return bool
   *   The status of the image.
   */
  public function isImageBuilderRoute(string $path) {
    $pathInfo = pathinfo($path);
    $pathData = explode('/', $pathInfo['dirname']);
    $mediaDirnameIndex = count($pathData) - 3;

    return isset($pathData[$mediaDirnameIndex]) && $pathData[$mediaDirnameIndex] === $this->getMediaDirname();
  }

  /**
   * Return the build image dirname.
   *
   * @return string
   *   THe media dirname.
   */
  public function getMediaDirname() {
    return $this->mediaTwigToolsConfig->getDirname();
  }

  /**
   * Return the no style name.
   *
   * @return string
   *   THe no style name.
   */
  public function getNoStyleName() {
    return $this->mediaTwigToolsConfig->getNoStyle();
  }

  /**
   * Generat the image and return the generator.
   *
   * @param string $path
   *   The image path.
   *
   * @return \Drupal\media_twig_tools\Generators\ImageGenerator
   *   The Image Generator.
   *
   * @throws \Exception
   */
  public function generateImage(string $path) {
    $generator = new ImageGenerator($this->getDataFromPath($path), $this);
    $generator->generate();

    return $generator;
  }

  /**
   * Return image builder data from image path.
   *
   * @param string $path
   *   The path.
   *
   * @return array
   *   The data.
   *
   * @throws \Exception
   */
  public function getDataFromPath($path) {
    if (!$this->isImageBuilderRoute($path)) {
      throw new \Exception('Not a media builder path');
    }

    $pathInfo = pathinfo($path);

    // File id.
    $nameData = explode('_', $pathInfo['basename']);
    $pathInfo['f'] = reset($nameData);
    $pathInfo['original_basename'] = implode('_', array_slice($nameData, 1));

    // Settings and options.
    $settings = explode('/', $pathInfo['dirname']);
    $count = count($settings);
    $pathInfo[MediaTwigTools::OPTION_RULES] = $this->getRulesFromPathString(end($settings));
    $pathInfo[MediaTwigTools::OPTION_STYLE] = $settings[$count - 2];

    return $pathInfo;
  }

  /**
   * Return the url image from file and settings.
   *
   * @param string $fileId
   *   The file id.
   * @param string $url
   *   The file url.
   * @param array $srcSettings
   *   The source settings.
   * @param array $srcSetSettings
   *   The source set settings.
   * @param array $options
   *   The options.
   * @param string $rule
   *   The rule.
   *
   * @return array|string|string[]|null
   *   The image path.
   */
  public function getImageUrlFromFileAndSettings($fileId, $url, array $srcSettings, array $srcSetSettings, array $options, $rule = 'reduce') {

    // Path data.
    $urlPath = pathinfo($url);
    $dirPath = pathinfo($urlPath['dirname']);

    // Rules.
    $rules = $this->getRulesData($srcSettings);

    // Get style.
    /** @var \Drupal\image\ImageStyleInterface $style */
    $style = isset($rules[MediaTwigTools::OPTION_STYLE]) ? $rules[MediaTwigTools::OPTION_STYLE] : NULL;
    if (!$style) {
      if (isset($srcSetSettings[MediaTwigTools::OPTION_STYLE])) {
        if ($srcSetSettings[MediaTwigTools::OPTION_STYLE] instanceof ImageStyleInterface) {
          $style = $srcSetSettings[MediaTwigTools::OPTION_STYLE];
        }
        else {
          $style = $this->mediaTwigToolsConfig->getImageStyleStorage()
            ->load($srcSetSettings[MediaTwigTools::OPTION_STYLE]);
        }
      }
      else {
        $style = NULL;
      }
    }

    return $this->replace(
      '@dir/@mediaDir/@style/@rules/@fid_@file.@extension',
      [
        '@dir'       => $dirPath['dirname'],
        '@mediaDir'  => $this->getMediaDirname(),
        '@style'     => $style ? $style->id() : $this->getNoStyleName(),
        '@rules'     => $this->getRulesDirnameFromRulesData($rules),
        '@file'      => $urlPath['filename'],
        '@extension' => $urlPath['extension'],
        '@fid'       => $fileId,
      ]
    );
  }

  /**
   * Scale and crop image.
   *
   * @param \Drupal\file\FileInterface $file
   *   THe file.
   * @param int $width
   *   THe width.
   * @param int $height
   *   The height.
   * @param array $options
   *   The options.
   *
   * @return array|string|string[]|null
   *   The url.
   */
  public function getScaledAndCroppedUrl(FileInterface $file, $width, $height, array $options = []) {
    $rules = [
      [
        "id"   => "image_scale_and_crop",
        "data" => [
          "width"  => $width,
          "height" => $height,
        ],
      ],
    ];

    return $this->getRuledUrlFromMedia($file, $rules, $options);
  }

  /**
   * Return ruled url.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file.
   * @param array $rules
   *   THe rules.
   * @param array $options
   *   The options.
   *
   * @return array|string|string[]|null
   *   The url.
   */
  public function getRuledUrlFromMedia(FileInterface $file, array $rules, array $options = []) {
    $srcSettings = [
      MediaTwigTools::OPTION_RULES => $rules,
    ];

    return $this->getImageUrlFromSettings($file->id(), $srcSettings, [], $options);
  }

}

==> media_twig_tools-1.0.5.tar.gz: media_twig_tools/src/Services/ImageBuilder.php <==
<?php

namespace Drupal\media_twig_tools\Services;

use Drupal\file\FileInterface;
use Drupal\image\ImageStyleInterface;
use Drupal\media_twig_tools\Config\MediaTwigToolsConfigInterface;
use Drupal\media_twig_tools\Generators\ImageGenerator;

/**
 * This class builds image from settings.
 *
 * @package Drupal\media_twig_tools\Services
 */
class ImageBuilder {

  /**
   * Nom du service.
   *
   * @const string
   */
  const SERVICE_NAME = 'media_twig_tools.image_builder';

  /**
   * Rules separator.
   *
   * @const string
   */
  const RULES_SEPARATOR = '@';

  /**
   * Conf.
   *
   * @var \Drupal\media_twig_tools\Config\MediaTwigToolsConfigInterface
   */
  protected $mediaTwigToolsConfig;

  /**
   * ImageBuilder constructor.
   *
   * @param \Drupal\media_twig_tools\Config\MediaTwigToolsConfigInterface $mediaTwigToolsConfig
   *   The config.
   */
  public function __construct(MediaTwigToolsConfigInterface $mediaTwigToolsConfig) {
    $this->mediaTwigToolsConfig = $mediaTwigToolsConfig;
  }

  /**
   * Retourne le singleton.
   *
   * @return static
   *   Le singleton.
   */
  public static function me() {
    return \Drupal::service(static::SERVICE_NAME);
  }

  /**
   * Return the url image from settings.
   *
   * @param string $fileId
   *   The file id.
   * @param array $srcSettings
   *   The source settings.
   * @param array $srcSetSettings
   *   The source set settings.
   * @param array $options
   *   The options.
   * @param string $rule
   *   The rule.
   *
   * @return array|string|string[]|null
   *   The image path.
   */
  public function getImageUrlFromSettings($fileId, array $srcSettings, array $srcSetSettings, array $options, $rule = 'reduce') {
    $imgUrl = NULL;

    // Get file.
    if ($file = $this->mediaTwigToolsConfig->getFileStorage()
      ->load(intval($fileId))) {

      $url = file_create_url($file->getFileUri());
      if (isset($options[MediaTwigTools::OPTION_RELATIVE]) && $options[MediaTwigTools::OPTION_RELATIVE]) {
        $url = file_url_transform_relative($url);
      }

      $imgUrl = $this->getImageUrlFromFileAndSettings($file->id(), $url, $srcSettings, $srcSetSettings, $options, $rule);
    }

    return $imgUrl;
  }

  /**
   * Quick replace.
   *
   * @param string $source
   *   The source string.
   * @param array $replace
   *   The replacement [$needle => $replacement].
   *
   * @return array|string|string[]
   *   The string replaced.
   */
  public function replace($source, array $replace) {
    return str_replace(
      array_keys($replace),
      array_values($replace),
      $source,
    );
  }

  /**
   * Return the rules from the dirname.
   *
   * @param string $path
   *   THe path.
   *
   * @return array
   *   The rules.
   */
  public function getRulesFromPathString($path) {
    $rules = [];
    foreach (explode(static::RULES_SEPARATOR, $path) as $data) {
      // Split letters and numbers.
      $split = preg_split("/(,?\s+)|((?<=[a-z])(?=\d))|((?<=\d)(?=[a-z]))/i", $data);

      // Associative array.
      if (is_numeric(reset($split))) {
        $key = 1;
        $delta = -1;
      }
      else {
        $key = 0;
        $delta = 1;
      }
      $rule = [
        'id'   => array_pop($split),
        'data' => [],
      ];
      for ($i = $key; $i <= count($split) - $delta; $i += 2) {
        if (isset($split[$i + $delta]) && isset($split[$i])) {
          $rule['data'][$split[$i]] = $split[$i + $delta];
        }
      }

      $rules[] = $rule;
    }

    return $rules;
  }

  /**
   * Return rules data.
   *
   * @param array $srcSettings
   *   The src settings.
   *
   * @return array
   *   The rules data.
   */
  public function getRulesData(array $srcSettings) {

    if (isset($srcSettings[MediaTwigTools::OPTION_STYLE])
      && $style = $this->mediaTwigToolsConfig->getImageStyleStorage()
        ->load($srcSettings[MediaTwigTools::OPTION_STYLE])) {
      $srcSettings[MediaTwigTools::OPTION_STYLE] = $style;
    }
    else {
      unset($srcSettings[MediaTwigTools::OPTION_STYLE]);
    }

    return $srcSettings;
  }

  /**
   * Return the dirname according to rules data.
   *
   * @param array $rulesData
   *   The rules data.
   *
   * @return string
   *   The rules dirname.
   */
  public function getRulesDirnameFromRulesData(array $rulesData) {
    $rules = [];
    foreach ($rulesData[MediaTwigTools::OPTION_RULES] as $weight => $rule) {
      $rules[$weight] = '';
      foreach ($rule['data'] as $key => $value) {
        $rules[$weight] .= $key . $value;
      }
      $rules[$weight] .= $rule['id'];
    }

    return implode(static::RULES_SEPARATOR, $rules);
  }

  /**
   * Return true if $path is an image builder route.
   *
   * @param string $path
   *   THe path.
   *
   * @return bool
   *   The status of the image.
   */
  public function isImageBuilderRoute(string $path) {
    $pathInfo = pathinfo($path);
    $pathData = explode('/', $pathInfo['dirname']);
    $mediaDirnameIndex = count($pathData) - 3;

    return isset($pathData[$mediaDirnameIndex]) && $pathData[$mediaDirnameIndex] === $this->getMediaDirname();
  }

  /**
   * Return the build image dirname.
   *
   * @return string
   *   THe media dirname.
   */
  public function getMediaDirname() {
    return $this->mediaTwigToolsConfig->getDirname();
  }

  /**
   * Return the no style name.
   *
   * @return string
   *   THe no style name.
   */
  public function getNoStyleName() {
    return $this->mediaTwigToolsConfig->getNoStyle();
  }

  /**
   * Generat the image and return the generator.
   *
   * @param string $path
   *   The image path.
   *
   * @return \Drupal\media_twig_tools\Generators\ImageGenerator
   *   The Image Generator.
   *
   * @throws \Exception
   */
  public function generateImage(string $path) {
    $generator = new ImageGenerator($this->getDataFromPath($path), $this);
    $generator->generate();

    return $generator;
  }

  /**
   * Return image builder data from image path.
   *
   * @param string $path
   *   The path.
   *
   * @return array
   *   The data.
   *
   * @throws \Exception
   */
  public function getDataFromPath($path) {
    if (!$this->isImageBuilderRoute($path)) {
      throw new \Exception('Not a media builder path');
    }

    $pathInfo = pathinfo($path);

    // File id.
    $nameData = explode('_', $pathInfo['basename']);
    $pathInfo['f'] = reset($nameData);
    $pathInfo['original_basename'] = implode('_', array_slice($nameData, 1));

    // Settings and options.
    $settings = explode('/', $pathInfo['dirname']);
    $count = count($settings);
    $pathInfo[MediaTwigTools::OPTION_RULES] = $this->getRulesFromPathString(end($settings));
    $pathInfo[MediaTwigTools::OPTION_STYLE] = $settings[$count - 2];

    return $pathInfo;
  }

  /**
   * Return the url image from file and settings.
   *
   * @param string $fileId
   *   The file id.
   * @param string $url
   *   The file url.
   * @param array $srcSettings
   *   The source settings.
   * @param array $srcSetSettings
   *   The source set settings.
   * @param array $options
   *   The options.
   * @param string $rule
   *   The rule.
   *
   * @return array|string|string[]|null
   *   The image path.
   */
  public function getImageUrlFromFileAndSettings($fileId, $url, array $srcSettings, array $srcSetSettings, array $options, $rule = 'reduce') {

    // Path data.
    $urlPath = pathinfo($url);
    $dirPath = pathinfo($urlPath['dirname']);

    // Rules.
    $rules = $this->getRulesData($srcSettings);

    // Get style.
    /** @var \Drupal\image\ImageStyleInterface $style */
    $style = isset($rules[MediaTwigTools::OPTION_STYLE]) ? $rules[MediaTwigTools::OPTION_STYLE] : NULL;
    if (!$style) {
      if (isset($srcSetSettings[MediaTwigTools::OPTION_STYLE])) {
        if ($srcSetSettings[MediaTwigTools::OPTION_STYLE] instanceof ImageStyleInterface) {
          $style = $srcSetSettings[MediaTwigTools::OPTION_STYLE];
        }
        else {
          $style = $this->mediaTwigToolsConfig->getImageStyleStorage()
            ->load($srcSetSettings[MediaTwigTools::OPTION_STYLE]);
        }
      }
      else {
        $style = NULL;
      }
    }

    return $this->replace(
      '@dir/@mediaDir/@style/@rules/@fid_@file.@extension',
      [
        '@dir'       => $dirPath['dirname'],
        '@mediaDir'  => $this->getMediaDirname(),
        '@style'     => $style ? $style->id() : $this->getNoStyleName(),
        '@rules'     => $this->getRulesDirnameFromRulesData($rules),
        '@file'      => $urlPath['filename'],
        '@extension' => $urlPath['extension'],
        '@fid'       => $fileId,
      ]
    );
  }

  /**
   * Scale and crop image.
   *
   * @param \Drupal\file\FileInterface $file
   *   THe file.
   * @param int $width
   *   THe width.
   * @param int $height
   *   The height.
   * @param array $options
   *   The options.
   *
   * @return array|string|string[]|null
   *   The url.
   */
  public function getScaledAndCroppedUrl(FileInterface $file, $width, $height, array $options = []) {
    $rules = [
      [
        "id"   => "image_scale_and_crop",
        "data" => [
          "width"  => $width,
          "height" => $height,
        ],
      ],
    ];

    return $this->getRuledUrlFromMedia($file, $rules, $options);
  }

  /**
   * Return ruled url.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file.
   * @param array $rules
   *   THe rules.
   * @param array $options
   *   The options.
   *
   * @return array|string|string[]|null
   *   The url.
   */
  public function getRuledUrlFromMedia(FileInterface $file, array $rules, array $options = []) {
    $srcSettings = [
      MediaTwigTools::OPTION_RULES => $rules,
    ];

    return $this->getImageUrlFromSettings($file->id(), $srcSettings, [], $options);
  }

}
